@@ -1,13 +1,29 @@
+/*
+ * hfsutils - tools for reading and writing Macintosh HFS volumes
+ * Copyright (C) 1996, 1997 Robert Leslie
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
+
 # include <stdio.h>
 # include <stdlib.h>
 # include <string.h>
 # include <errno.h>
 # include <unistd.h>
 
-# ifdef DMALLOC
-# include <dmalloc.h>
-# endif
-
+# include "hfs.h"
 # include "hcwd.h"
 
 # define STATEFNAME  ".hcwd"
@@ -46,7 +62,7 @@ int addent(mountent *ent)
 }
 
 /*
- * NAME:	hcwd_init()
+ * NAME:	hcwd->init()
  * DESCRIPTION:	read state file
  */
 int hcwd_init(void)
@@ -86,13 +102,11 @@ int hcwd_init(void)
       while (*ptr && *ptr != '\n' && *ptr != '\t')
 	++ptr;
 
+      entry.vname[0] = 0;
       if (*ptr == '\t')
 	{
 	  *ptr++ = 0;
-	  entry.vname = strdup(start);
-	  if (entry.vname == 0)
-	    return -1;
-
+	  strcpy(entry.vname, start);
 	  start = ptr;
 	}
 
@@ -152,7 +166,7 @@ int hcwd_init(void)
 }
 
 /*
- * NAME:	hcwd_finish()
+ * NAME:	hcwd->finish()
  * DESCRIPTION:	flush changes to state file
  */
 int hcwd_finish(void)
@@ -184,8 +198,6 @@ int hcwd_finish(void)
 
       for (ent = mounts; ent < mounts + nmounts; ++ent)
 	{
-	  if (ent->vname)
-	    free(ent->vname);
 	  if (ent->path)
 	    free(ent->path);
 	  if (ent->cwd)
@@ -202,7 +214,7 @@ int hcwd_finish(void)
 }
 
 /*
- * NAME:	hcwd_mounted()
+ * NAME:	hcwd->mounted()
  * DESCRIPTION:	register a mounted volume
  */
 int hcwd_mounted(char *vname, long vcrdate, char *path, int partno)
@@ -216,16 +228,13 @@ int hcwd_mounted(char *vname, long vcrdate, char *path, int partno)
 	{
 	  /* update entry */
 
-	  if (entry->vname)
-	    free(entry->vname);
 	  if (entry->cwd)
 	    free(entry->cwd);
 
-	  entry->vname   = strdup(vname);
+	  strcpy(entry->vname, vname);
 	  entry->vcrdate = vcrdate;
 	  entry->cwd     = strdup(":");
-	  if (entry->vname == 0 ||
-	      entry->cwd == 0)
+	  if (entry->cwd == 0)
 	    return -1;
 
 	  curvol = entry - mounts;
@@ -235,14 +244,13 @@ int hcwd_mounted(char *vname, long vcrdate, char *path, int partno)
 	}
     }
 
-  new.vname   = strdup(vname);
+  strcpy(new.vname, vname);
   new.vcrdate = vcrdate;
   new.path    = strdup(path);
   new.partno  = partno;
   new.cwd     = strdup(":");
 
-  if (new.vname == 0 ||
-      new.path  == 0 ||
+  if (new.path  == 0 ||
       new.cwd   == 0)
     return -1;
 
@@ -255,7 +263,42 @@ int hcwd_mounted(char *vname, long vcrdate, char *path, int partno)
 }
 
 /*
- * NAME:	hcwd_getvol()
+ * NAME:	hcwd->umounted()
+ * DESCRIPTION:	unregister a previously mounted volume
+ */
+int hcwd_umounted(int vol)
+{
+  mountent *entry;
+  int i;
+
+  if (vol < 0)
+    vol = curvol;
+
+  if (vol < 0 || vol >= nmounts)
+    return -1;
+
+  entry = &mounts[vol];
+
+  if (entry->path)
+    free(entry->path);
+  if (entry->cwd)
+    free(entry->cwd);
+
+  for (i = vol + 1; i < nmounts; ++i)
+    mounts[i - 1] = mounts[i];
+
+  --nmounts;
+
+  if (curvol > vol)
+    --curvol;
+  else if (curvol == vol)
+    curvol = -1;
+
+  return 0;
+}
+
+/*
+ * NAME:	hcwd->getvol()
  * DESCRIPTION:	return a mount entity
  */
 mountent *hcwd_getvol(int vol)
@@ -270,7 +313,7 @@ mountent *hcwd_getvol(int vol)
 }
 
 /*
- * NAME:	hcwd_setvol()
+ * NAME:	hcwd->setvol()
  * DESCRIPTION:	change the current volume
  */
 int hcwd_setvol(int vol)
@@ -288,7 +331,7 @@ int hcwd_setvol(int vol)
 }
 
 /*
- * NAME:	hcwd_setcwd()
+ * NAME:	hcwd->setcwd()
  * DESCRIPTION:	change the current working directory for a mount entity
  */
 int hcwd_setcwd(mountent *ent, char *newcwd)
@@ -299,8 +342,10 @@ int hcwd_setcwd(mountent *ent, char *newcwd)
   while (*path && *path != ':')
     ++path;
 
-  cwd = (*path == 0) ? strdup(":") : strdup(path);
+  memcpy(ent->vname, newcwd, path - newcwd);
+  ent->vname[path - newcwd] = 0;
 
+  cwd = (*path == 0) ? strdup(":") : strdup(path);
   if (cwd == 0)
     return -1;
 
